<?php
/**
 *
 * @copyright 2008 - https://www.clicshopping.org
 * @Brand : ClicShoppingAI(TM) at Inpi all right Reserved
 * @Licence GPL 2 & MIT
 * @Info : https://www.clicshopping.org/forum/trademark/
 *
 */

namespace ClicShopping\Apps\Configuration\TemplateEmail\Classes\ClicShoppingAdmin;

use ClicShopping\OM\HTTP;
use ClicShopping\OM\Registry;

class TemplateEmailAdmin
{
  /**
   * the name of the template
   *
   * @param int $template_email_id , $language_id
   * @param int $language_id
   * @return string $template_email_name['template_name'],  name.of the template email
   */
  public static function getTemplateEmailName(int $template_email_id, int $language_id): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    if (!$language_id) $language_id = $CLICSHOPPING_Language->getId();

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select template_email_name
                                                  from :table_template_email_description
                                                  where template_email_id = :template_email_id
                                                  and language_id = :language_id
                                                 ');
    $QtemplateEmail->bindInt(':template_email_id', (int)$template_email_id);
    $QtemplateEmail->bindInt(':language_id', (int)$language_id);

    $QtemplateEmail->execute();

    return $QtemplateEmail->value('template_email_name');
  }

  /**
   * the template email short description
   *
   * @param int $template_email_id , $language_id
   * @param int $language_id
   * @return string $template_email['template_short_description'],  the short description of the template email
   */
  public static function getTemplateEmailShortDescription(int $template_email_id, int $language_id): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    if (!$language_id) $language_id = $CLICSHOPPING_Language->getId();

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select template_email_short_description
                                                  from :table_template_email_description
                                                  where template_email_id = :template_email_id
                                                  and language_id = :language_id
                                                 ');
    $QtemplateEmail->bindInt(':template_email_id', (int)$template_email_id);
    $QtemplateEmail->bindInt(':language_id', (int)$language_id);

    $QtemplateEmail->execute();

    return $QtemplateEmail->value('template_email_short_description');
  }

  /**
   * the template email description who is sent
   *
   * @param int $template_email_id , $language_id
   * @param int $language_id
   * @return string $template_email['template_email_description'],  the description of the template email who is sent
   */
  public static function getTemplateEmailDescription(int $template_email_id, int $language_id): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    if (!$language_id) $language_id = $CLICSHOPPING_Language->getId();

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select template_email_description
                                                  from :table_template_email_description
                                                  where template_email_id = :template_email_id
                                                  and language_id = :language_id
                                                ');
    $QtemplateEmail->bindInt(':template_email_id', (int)$template_email_id);
    $QtemplateEmail->bindInt(':language_id', (int)$language_id);

    $QtemplateEmail->execute();

    return $QtemplateEmail->value('template_email_description');
  }

  /**
   * the footer of email
   *
   * @return string $template_email_footer,  the footer of the email template who is sent
   */

  public static function getTemplateEmailTextFooter(): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select te.template_email_variable,
                                                         ted.template_email_description
                                                  from :table_template_email te,
                                                       :table_template_email_description ted
                                                  where te.template_email_variable = :template_email_variable
                                                  and te.template_email_id = ted.template_email_id
                                                  and ted.language_id = :language_id
                                               ');
    $QtemplateEmail->bindValue(':template_email_variable', 'TEMPLATE_EMAIL_TEXT_FOOTER');
    $QtemplateEmail->bindInt(':language_id', (int)$CLICSHOPPING_Language->getId());


    $QtemplateEmail->execute();

    $template_email_footer = $QtemplateEmail->value('template_email_description');

    $keywords = array('/{{store_name}}/',
      '/{{store_owner_email_address}}/',
      '/{{http_shop}}/'
    );

    $replaces = [
      STORE_NAME,
      STORE_OWNER_EMAIL_ADDRESS,
      HTTP::getShopUrlDomain()
    ];


    $template_email_footer = preg_replace($keywords, $replaces, $template_email_footer);

    return $template_email_footer;
  }

  /**
   * the footer of the newsletter
   *
   * @return string $template_email_newsletter_ footer,  the footer of the newsletter email template who is sent
   */
  public static function getTemplateEmailNewsletterTextFooter(): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select te.template_email_variable,
                                                         ted.template_email_description
                                                  from :table_template_email te,
                                                       :table_template_email_description ted
                                                  where te.template_email_variable = :template_email_variable
                                                  and te.template_email_id = ted.template_email_id
                                                  and ted.language_id = :language_id
                                                ');
    $QtemplateEmail->bindValue(':template_email_variable', 'TEMPLATE_EMAIL_NEWSLETTER_TEXT_FOOTER');
    $QtemplateEmail->bindInt(':language_id', $CLICSHOPPING_Language->getId());


    $QtemplateEmail->execute();

    $template_email_newsletter_footer = $QtemplateEmail->value('template_email_description');

    $keywords = array('/{{store_name}}/',
      '/{{store_owner_email_address}}/',
      '/{{http_shop}}/'
    );

    $replaces = array(STORE_NAME,
      STORE_OWNER_EMAIL_ADDRESS,
      HTTP::getShopUrlDomain()
    );

    $template_email_newsletter_footer = preg_replace($keywords, $replaces, $template_email_newsletter_footer);

    return $template_email_newsletter_footer;
  }

  /**
   * the signature of email
   *
   * @return string $template_email_signature,  the signature of the email template who is sent
   */
  public static function getTemplateEmailSignature(): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select te.template_email_variable,
                                                           ted.template_email_description
                                                    from :table_template_email te,
                                                         :table_template_email_description ted
                                                    where te.template_email_variable = :template_email_variable
                                                    and te.template_email_id = ted.template_email_id
                                                    and ted.language_id = :language_id
                                                  ');
    $QtemplateEmail->bindValue(':template_email_variable', 'TEMPLATE_EMAIL_SIGNATURE');
    $QtemplateEmail->bindInt(':language_id', $CLICSHOPPING_Language->getId());


    $QtemplateEmail->execute();


    $template_email_signature = $QtemplateEmail->value('template_email_description');

    $keywords = array('/{{store_name}}/',
      '/{{store_owner_email_address}}/',
      '/{{http_shop}}/'

    );

    $replaces = [
      STORE_NAME,
      STORE_OWNER_EMAIL_ADDRESS,
      HTTP::getShopUrlDomain()
    ];

    $template_email_signature = preg_replace($keywords, $replaces, $template_email_signature);

    return $template_email_signature;
  }

  /**
   * the template email welcome admin who is sent
   *
   * @return string $template_email_welcome_admin,  the description of the template email welcome admin who is sent
   */
  public static function getTemplateEmailWelcomeAdmin(): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select te.template_email_variable,
                                                         ted.template_email_description
                                                  from :table_template_email te,
                                                       :table_template_email_description ted
                                                  where te.template_email_variable = :template_email_variable
                                                  and te.template_email_id = ted.template_email_id
                                                  and ted.language_id = :language_id
                                                ');
    $QtemplateEmail->bindValue(':template_email_variable', 'TEMPLATE_EMAIL_WELCOME_ADMIN');
    $QtemplateEmail->bindInt(':language_id', $CLICSHOPPING_Language->getId());


    $QtemplateEmail->execute();

    $template_email_welcome_admin = $QtemplateEmail->value('template_email_description');

    $keywords = [
      '/{{store_name}}/',
      '/{{store_owner_email_address}}/',
      '/{{http_shop}}/'
    ];

    $replaces = [
      STORE_NAME,
      STORE_OWNER_EMAIL_ADDRESS,
      HTTP::getShopUrlDomain()
    ];

    $template_email_welcome_admin = preg_replace($keywords, $replaces, $template_email_welcome_admin);

    return $template_email_welcome_admin;
  }

  /**
   * the template email coupon who is sent
   *
   * @return string $template_email_coupon_admin,  the description of the template email coupon who is sent
   */
  public static function getTemplateEmailCouponAdmin(): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select te.template_email_variable,
                                                         ted.template_email_description
                                                  from :table_template_email te,
                                                       :table_template_email_description ted
                                                  where te.template_email_variable = :template_email_variable
                                                  and te.template_email_id = ted.template_email_id
                                                  and ted.language_id = :language_id
                                                ');
    $QtemplateEmail->bindValue(':template_email_variable', 'TEMPLATE_EMAIL_TEXT_COUPON');
    $QtemplateEmail->bindInt(':language_id', $CLICSHOPPING_Language->getId());


    $QtemplateEmail->execute();

    $template_email_coupon_admin = $QtemplateEmail->value('template_email_description');

    $keywords = [
      '/{{store_name}}/',
      '/{{store_owner_email_address}}/',
      '/{{http_shop}}/'
    ];

    $replaces = [
      STORE_NAME,
      STORE_OWNER_EMAIL_ADDRESS,
      HTTP::getShopUrlDomain()
    ];

    $template_email_coupon_admin = preg_replace($keywords, $replaces, $template_email_coupon_admin);

    return $template_email_coupon_admin;
  }

  /**
   * the template order intro command who is sent
   *
   * @return string $template_email_intro_command,  the description of the template email order intro command who is sent
   */

  public static function getTemplateEmailIntroCommand(): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select te.template_email_variable,
                                                           ted.template_email_description
                                                    from :table_template_email te,
                                                         :table_template_email_description ted
                                                    where te.template_email_variable = :template_email_variable
                                                    and te.template_email_id = ted.template_email_id
                                                    and ted.language_id = :language_id
                                                  ');
    $QtemplateEmail->bindValue(':template_email_variable', 'TEMPLATE_EMAIL_INTRO_COMMAND');
    $QtemplateEmail->bindInt(':language_id', $CLICSHOPPING_Language->getId());


    $QtemplateEmail->execute();

    $template_email_intro_command = $QtemplateEmail->value('template_email_description');

    $keywords = [
      '/{{store_name}}/',
      '/{{store_owner_email_address}}/',
      '/{{http_shop}}/'
    ];

    $replaces = [STORE_NAME,
      STORE_OWNER_EMAIL_ADDRESS,
      HTTP::getShopUrlDomain()
    ];

    $template_email_intro_command = preg_replace($keywords, $replaces, $template_email_intro_command);

    return $template_email_intro_command;
  }

  /**
   * @param string $string
   * @return string
   */
  public static function getTemplateEmailDescriptionByTemplateVariable(string $string): string
  {
    $CLICSHOPPING_Db = Registry::get('Db');
    $CLICSHOPPING_Language = Registry::get('Language');

    $QtemplateEmail = $CLICSHOPPING_Db->prepare('select te.template_email_variable,
                                                           ted.template_email_description
                                                    from :table_template_email te,
                                                         :table_template_email_description ted
                                                    where te.template_email_variable = :template_email_variable
                                                    and te.template_email_id = ted.template_email_id
                                                    and ted.language_id = :language_id
                                                  ');
    $QtemplateEmail->bindValue(':template_email_variable', $string);
    $QtemplateEmail->bindInt(':language_id', $CLICSHOPPING_Language->getId());


    $QtemplateEmail->execute();

    return $QtemplateEmail->value('template_email_description');
  }
}